@@ -25,17 +25,16 @@
 #include "viewportwidget.h"
 #include "functionedit.h"
 #ifdef HAVE_OPENGL
-#	include "graph3d.h"
+#	include <analitzagui//plotsview3d.h>
 #endif
 
 #include <analitzagui/operatorsmodel.h>
-#include <analitzagui/functionfactory.h>
-
-#include <analitzagui/functionsmodel.h>
-#include <analitzagui/functionsview.h>
 #include <analitzagui/expressionedit.h>
-#include <analitzagui/graph2d.h>
 #include <analitzagui/variablesmodel.h>
+#include <analitzagui/plotsview2d.h>
+#include <analitzaplot/plotsmodel.h>
+#include <analitzaplot/functiongraph.h>
+#include <analitzaplot/planecurve.h>
 #include <analitza/variables.h>
 #include <analitza/value.h>
 
@@ -56,6 +55,7 @@
 #include <KRecentFilesAction>
 #include <KApplication>
 #include <kabstractfilewidget.h>
+#include <analitzaplot/plotsfactory.h>
 
 class Add2DOption : public InlineOptions
 {
@@ -65,11 +65,8 @@ class Add2DOption : public InlineOptions
 		{}
 		
 		virtual QString id() const { return "add2d"; }
-		virtual bool matchesExpression(const Analitza::Expression& exp, const Analitza::ExpressionType& functype) const
-		{
-			bool cont = FunctionFactory::self()->contains(exp.bvarList());
-				
-			return cont && functype.canReduceTo(FunctionFactory::self()->type(exp.bvarList()));
+		virtual bool matchesExpression(const Analitza::Expression& exp, const Analitza::ExpressionType& functype) const {
+			return Analitza::PlotsFactory::self()->requestPlot(exp, Analitza::Dim2D).canDraw();
 		}
 
 		virtual QString caption() const { return i18n("Plot 2D"); }
@@ -91,7 +88,7 @@ class Add3DOption : public InlineOptions
 		virtual QString id() const { return "add3d"; }
 		virtual bool matchesExpression(const Analitza::Expression& exp, const Analitza::ExpressionType& functype) const
 		{
-			return Graph3D::checkExpression(exp.bvarList(), functype);
+			return Analitza::PlotsFactory::self()->requestPlot(exp, Analitza::Dim3D).canDraw();
 		}
 
 		virtual QString caption() const { return i18n("Plot 3D"); }
@@ -132,7 +129,7 @@ KAlgebra::KAlgebra(QWidget *parent) : KMainWindow(parent)
 	c_dock_vars->setFeatures(QDockWidget::DockWidgetFloatable|QDockWidget::DockWidgetMovable);
 	addDockWidget(Qt::RightDockWidgetArea, c_dock_vars);
 	
-	c_varsModel=new VariablesModel(c_results->analitza()->variables());
+	c_varsModel=new Analitza::VariablesModel(c_results->analitza()->variables());
 	c_varsModel->setEditable(false);
 	
 	c_variables = new QTreeView(c_dock_vars);
@@ -142,7 +139,7 @@ KAlgebra::KAlgebra(QWidget *parent) : KMainWindow(parent)
 	c_variables->setSelectionBehavior(QAbstractItemView::SelectRows);
 	c_variables->setSelectionMode(QAbstractItemView::SingleSelection);
 	
-	c_exp = new ExpressionEdit(console);
+	c_exp = new Analitza::ExpressionEdit(console);
 	c_exp->setAnalitza(c_results->analitza());
 	c_exp->setExamples(QStringList() << "square:=x->x**2" << "fib:=n->piecewise { eq(n,0)?0, eq(n,1)?1, ?fib(n-1)+fib(n-2) }");
 	c_dock_vars->setWidget(c_variables);
@@ -188,16 +185,19 @@ KAlgebra::KAlgebra(QWidget *parent) : KMainWindow(parent)
 	//////EOConsola
 	
 	//////2D Graph
-	b_funcsModel=new FunctionsModel(this);
+	b_funcsModel=new Analitza::PlotsModel(this);
 	
-	m_graph2d = new Graph2D(b_funcsModel, m_tabs);
+	m_graph2d = new Analitza::PlotsView2D(m_tabs);
+	m_graph2d->setTicksShown(0);
+	m_graph2d->setModel(b_funcsModel);
 	
 	b_dock_funcs = new QDockWidget(i18n("Functions"), this);
 	b_tools = new QTabWidget(b_dock_funcs);
 	b_tools->setTabPosition(QTabWidget::South);
 	addDockWidget(Qt::RightDockWidgetArea, b_dock_funcs);
 	
-	b_funcs = new FunctionsView(b_tools);
+	b_funcs = new QTreeView(b_tools);
+	b_funcs->setRootIsDecorated(false);
 	b_funcs->setModel(b_funcsModel);
 	b_funcs->header()->resizeSections(QHeaderView::ResizeToContents);
 	b_funcs->setSelectionMode(QAbstractItemView::SingleSelection);
@@ -211,7 +211,7 @@ KAlgebra::KAlgebra(QWidget *parent) : KMainWindow(parent)
 	b_tools->addTab(b_funced, KIcon("list-add"), i18n("&Add"));
 	
 	QTableView* b_varsView=new QTableView(b_tools);
-	b_varsModel=new VariablesModel(b_funced->variables());
+	b_varsModel=new Analitza::VariablesModel(b_funced->variables());
 	b_varsView->setModel(b_varsModel);
 	b_varsView->setShowGrid(false);
 	b_varsView->verticalHeader()->hide();
@@ -281,10 +281,13 @@ KAlgebra::KAlgebra(QWidget *parent) : KMainWindow(parent)
 #ifdef HAVE_OPENGL
 	QWidget *tridim = new QWidget(m_tabs);
 	QVBoxLayout *t_layo = new QVBoxLayout(tridim);
-	t_exp = new ExpressionEdit(tridim);
+	t_exp = new Analitza::ExpressionEdit(tridim);
 	t_exp->setExamples(QStringList() << "sin x+sin y" << "(x,y)->x" << "x*y");
 	t_exp->setAns("x");
-	m_graph3d = new Graph3D(tridim);
+	t_model3d = new Analitza::PlotsModel(this);
+	m_graph3d = new Analitza::PlotsView3D(tridim);
+	m_graph3d->setModel(t_model3d);
+	m_graph3d->setUseSimpleRotation(true);
 	
 	tridim->setLayout(t_layo);
 	m_tabs->addTab(tridim, i18n("&3D Graph"));
@@ -292,16 +295,14 @@ KAlgebra::KAlgebra(QWidget *parent) : KMainWindow(parent)
 	t_layo->addWidget(t_exp);
 	
 	connect(t_exp,  SIGNAL(returnPressed()), this, SLOT(new_func3d()));
-	connect(m_graph3d, SIGNAL(status(QString)), this, SLOT(changeStatusBar(QString)));
 	c_results->addOptionsObserver(new Add3DOption(this));
 	
 	////////menu
 	t_menu = menuBar()->addMenu(i18n("3D &Graph"));
 	QAction* t_actions[5];
-	t_actions[0] = t_menu->addAction(i18n("&Transparency"), this, SLOT(toggleTransparency()));
 	t_menu->addAction(KStandardAction::save(this, SLOT(save3DGraph()), this));
 	t_menu->addAction(KIcon("zoom-original"), i18n("&Reset View"), m_graph3d, SLOT(resetView()));
-	t_menu->addSeparator()->setText(i18n("Type"));
+	t_menu->addSeparator();
 	t_actions[2] = t_menu->addAction(i18n("Dots"), this, SLOT(set_dots()));
 	t_actions[3] = t_menu->addAction(i18n("Lines"), this, SLOT(set_lines()));
 	t_actions[4] = t_menu->addAction(i18n("Solid"), this, SLOT(set_solid()));
@@ -311,7 +312,6 @@ KAlgebra::KAlgebra(QWidget *parent) : KMainWindow(parent)
 	t_type->addAction(t_actions[3]);
 	t_type->addAction(t_actions[4]);
 	
-	t_actions[0]->setCheckable(true);
 	t_actions[2]->setCheckable(true);
 	t_actions[3]->setCheckable(true);
 	t_actions[4]->setCheckable(true);
@@ -350,11 +350,12 @@ KAlgebra::KAlgebra(QWidget *parent) : KMainWindow(parent)
 	//Ego's reminder
 	KHelpMenu* help = new KHelpMenu(this, KGlobal::mainComponent().aboutData());
 	menuBar()->addMenu(help->menu());
-	
-	connect(b_funcsModel, SIGNAL(functionModified(QString,Analitza::Expression)),
-			c_results, SLOT(modifyVariable(QString,Analitza::Expression)));
-	connect(b_funcsModel, SIGNAL(functionRemoved(QString)),
-			c_results, SLOT(removeVariable(QString)));
+
+#warning TODO: Port to PlotsModel
+// 	connect(b_funcsModel, SIGNAL(functionModified(QString,Analitza::Expression)),
+// 			c_results, SLOT(modifyVariable(QString,Analitza::Expression)));
+// 	connect(b_funcsModel, SIGNAL(functionRemoved(QString)),
+// 			c_results, SLOT(removeVariable(QString)));
 	
 	connect(m_tabs, SIGNAL(currentChanged(int)), this, SLOT(tabChanged(int)));
 	tabChanged(0);
@@ -391,26 +392,28 @@ void KAlgebra::add2D(const Analitza::Expression& exp)
 {
 	qDebug() << "adding" << exp.toString();
 	
-	Function f(b_funcsModel->freeId(), exp, c_results->analitza()->variables(), QPen(randomFunctionColor()), 6, 0);
-	b_funcsModel->addFunction(f);
+	Analitza::PlotItem* curve = Analitza::PlotsFactory::self()->requestPlot(exp, Analitza::Dim2D).create(randomFunctionColor(), b_funcsModel->freeId(),
+																		   c_results->analitza()->variables());
+	b_funcsModel->addPlot(curve);
 	
 	m_tabs->setCurrentIndex(1);
 }
 
 void KAlgebra::new_func()
 {
-	Function f=b_funced->createFunction();
+	Analitza::FunctionGraph* f=b_funced->createFunction();
 	
 	if(b_funced->editing()) {
-		b_funcsModel->editFunction(f.name(), f);
+		QModelIndex idx = b_funcsModel->indexForName(f->name());
+		b_funcsModel->updatePlot(idx.row(), f);
 	} else {
-		b_funcsModel->addFunction(f);
+		b_funcsModel->addPlot(f);
 	}
 	
 	b_funced->setEditing(false);
 	b_funced->clear();
 	b_tools->setCurrentIndex(0);
-	b_funcs->setCurrentIndex(b_funcsModel->indexForId(f.name()));
+	b_funcs->setCurrentIndex(b_funcsModel->indexForName(f->name()));
 	m_graph2d->setFocus();
 }
 
@@ -513,29 +516,34 @@ void KAlgebra::set_res_vfine()	{ b_funcsModel->setResolution(3328);}
 void KAlgebra::new_func3d()
 {
 #ifdef HAVE_OPENGL
-	m_graph3d->setFunc(t_exp->expression());
-	m_graph3d->setFocus();
+	Analitza::Expression exp = t_exp->expression();
+	Analitza::PlotBuilder plot = Analitza::PlotsFactory::self()->requestPlot(exp, Analitza::Dim3D);
+	if(plot.canDraw()) {
+		t_model3d->clear();
+		t_model3d->addPlot(plot.create(Qt::yellow, "func3d", c_results->analitza()->variables()));
+	} else
+		changeStatusBar(i18n("Errors: %1", plot.errors().join(i18n(", "))));
 #endif
 }
 
 void KAlgebra::set_dots()
 {
 #ifdef HAVE_OPENGL
-	m_graph3d->setMethod(Graph3D::Dots);
+    m_graph3d->setPlotStyle(Analitza::Dots);
 #endif
 }
 
 void KAlgebra::set_lines()
 {
 #ifdef HAVE_OPENGL
-	m_graph3d->setMethod(Graph3D::Lines);
+    m_graph3d->setPlotStyle(Analitza::Wired);
 #endif
 }
 
 void KAlgebra::set_solid()
 {
 #ifdef HAVE_OPENGL
-	m_graph3d->setMethod(Graph3D::Solid);
+    m_graph3d->setPlotStyle(Analitza::Solid);
 #endif
 }
 
@@ -543,15 +551,10 @@ void KAlgebra::save3DGraph()
 {
 #ifdef HAVE_OPENGL
 	QString path = KFileDialog::getSaveFileName(KUrl(), i18n("*.png|PNG File"), this, QString(), KFileDialog::ConfirmOverwrite);
-	if(!path.isEmpty())
-		m_graph3d->toPixmap().save(path, "PNG");
-#endif
-}
-
-void KAlgebra::toggleTransparency()
-{
-#ifdef HAVE_OPENGL
-	m_graph3d->setTransparency(!m_graph3d->transparency());
+	if(!path.isEmpty()) {
+		QPixmap px = m_graph3d->renderPixmap();
+		px.save(path);
+	}
 #endif
 }
 
@@ -576,7 +579,7 @@ void KAlgebra::saveGraph()
 		QString filename = dialog->selectedFile();
 		
 		bool isSvg = filename.endsWith(".svg") || (!filename.endsWith(".png") && filter.mid(2, 3)=="svg");
-		Graph2D::Format f = isSvg ? Graph2D::PNG : Graph2D::SVG;
+		Analitza::PlotsView2D::Format f = isSvg ? Analitza::PlotsView2D::PNG : Analitza::PlotsView2D::SVG;
 		m_graph2d->toImage(filename, f);
 	}
 	delete dialog;
@@ -652,7 +655,7 @@ void KAlgebra::valueChanged()
 {
 	//FIXME: Should only repaint the affected ones.
 	if(b_funcsModel->rowCount()>0)
-		m_graph2d->updateFunctions(b_funcsModel->index(0,0), b_funcsModel->index(b_funcsModel->rowCount()-1,0));
+		m_graph2d->updateFunctions(QModelIndex(), 0, b_funcsModel->rowCount()-1);
 }
 
 void KAlgebra::varsContextMenu(const QPoint& p)
@@ -674,7 +677,8 @@ void KAlgebra::varsContextMenu(const QPoint& p)
 void KAlgebra::add3D(const Analitza::Expression& exp)
 {
 #ifdef HAVE_OPENGL
-	m_graph3d->setFunc(exp);
+	t_model3d->clear();
+	t_model3d->addPlot(Analitza::PlotsFactory::self()->requestPlot(exp, Analitza::Dim3D).create(Qt::yellow, "func3d_console", c_results->analitza()->variables()));
 	m_tabs->setCurrentIndex(2);
 #endif
 }
